require_relative 'player'
require_relative 'die'
require_relative 'game_turn'
require_relative 'treasure_trove'
require 'pry'

module StudioGame
  class Game

    attr_reader :title

    def initialize(title)
      @title = title
      @players = []
    end

    def add_player(p_name)
      @players << p_name
    end

    def play(rounds)
      puts "There are #{@players.size} players in #{@title}."
      puts @players
      treasures = TreasureTrove::TREASURES
      puts "\nThere are #{treasures.size} treasures to be found:"
      treasures.each { |treasure| puts "A #{treasure.name} is worth #{treasure.points} points." }

      1.upto(rounds) do |count|
        puts "\nRound #{count}:"
        @players.each do |player|
          GameTurn.take_turn(player)
        end
      end
    end

    def print_stats
      puts "\n#{title}'s Stats:"

      @players.sort.each do |player|
        puts "\n#{player.player_name}'s point totals:"
        player.each_found_treasure do |treasure|
          puts "#{treasure.points} total #{treasure.name} points."
        end
        puts "#{player.points} grand total points."
      end

      strong, weak = @players.partition {|player| player.strong?}

      puts "\nStrong:"
      # strong.each {|player| print_name_and_health(player)}
      # puts strong
      if (strong)
        print_name_and_health(strong)
      end

      puts "\nWeak:"
      # weak.each {|player| print_name_and_health(player)}
      # puts weak
      if (weak)
        print_name_and_health(weak)
      end
      # binding.pry
      #sorted_players = @players.sort {|p1, p2| p2.health <=> p1.health}
      puts "\n#{title} High Scores:"
      #@players.sort.each {|player| puts player.player_name.ljust(10,'.') + "#{player.score}"}
      @players.sort.each {|player| puts high_score_entry(player)}
      #weak.each {|player| print_name_and_health(player)}

    end

    def print_name_and_health(player)
      player.sort.each {|player| puts player.player_name.ljust(20,'.') + "#{player.health}"}
    end

    #stuck here. I do not know why it is not saving the name.
    #whenever I run it says @player_name=nil

    def load_players(file_name)
      File.readlines(file_name).each do |line|
        # name, health = line.split(',')
        # new_player = Player.new(name, health.to_i)
        # add_player(new_player)
        add_player(Player.from_csv(line))
      end
    end

    def save_high_scores(file_name="high_scores.txt")
      File.open(file_name, "w") do |file|
        @players.sort.each do |player|
          # formatted_name = player.player_name.ljust(20, '.')
          # file.puts "#{formatted_name} #{player.score}"
          file.puts high_score_entry(player)
        end
      end
    end

    def high_score_entry(player)
      formatted_name = player.player_name.ljust(20, '.')
      "#{formatted_name} #{player.score}"
    end
  end
end
