require_relative 'treasure_trove'
require_relative 'playable'

module StudioGame

  class Player

    include Playable
    attr_accessor :player_name, :health


    def initialize(player_name, health=100)
      @player_name = player_name.capitalize
      @health = health
      @found_treasures = Hash.new(0)
    end

    def to_s
      "I'm #{@player_name} with health = #{@health}, points = #{points()}, and score = #{score()}."
    end

    def score
      #@player_name.length + @health
      @health + points()
    end

    def player_name=(new_name)
      self.player_name = new_name.capitalize!
    end

    def <=>(other_player)
      other_player.score <=> score() && other_player.score <=> score()
    end

    def found_treasure(treasure)
      @found_treasures[treasure.name] += treasure.points
      puts "#{player_name} found a #{treasure.name} worth #{treasure.points} points."
      puts "#{player_name}'s treasures: #{@found_treasures}"
    end

    def points
      @found_treasures.values.reduce(0,:+)
    end

    def each_found_treasure
      @found_treasures.each do |name, points|
        yield Treasure.new(name, points)
      end
    end

    def self.from_csv(line)
      name, health = line.split(',')
      Player.new(name, Integer(health))
    end
  end
end

if __FILE__ == $0
  player = Player.new("moe")
  puts player.player_name
  puts player.health
  player.w00t
  puts player.health
  player.blam
  puts player.health
end
